#!/usr/bin/env python3
# -*- coding: utf-8 -*-   
# ###################################################################################################
# 
# This is a quick and dirty script written by Michael Gallagher (michael.r.gallagher@noaa.gov)
# that will take a logger csv file and get all rows for a specific day and put them into a new
# file. It makes no assumptions about file contents, copying all columns, etc, *EXCEPT* we
# assume that the header is 5 lines total _and_ that the date of each entry is the first column
# in the files.
#
# This can be run for specific dates, or today. If you don't supply a date as an argument
# it will assume that you wanted to parse out yesterdays data. Can be run as follows:
# 
# python3 create_daily_level0.py --datetoparse 20220903 --savedir /tmp/
#
# OR shorter syntax
#
# ./create_daily_level0.py -d 20220903 -s /tmp/
#
# ###################################################################################################

import os, inspect, argparse, time, gc

import socket 

# from multiprocessing import Process as P
# from multiprocessing import Queue   as Q
 
import numpy  as np
import pandas as pd
import xarray as xr

pd.options.mode.use_inf_as_na = True # no inf values anywhere

from datetime  import datetime, timedelta
from numpy     import sqrt
from netCDF4   import Dataset

import warnings; warnings.filterwarnings(action='ignore') # vm python version problems, cleans output....

# just in case... avoids some netcdf nonsense involving the default file locking across mounts
os.environ['HDF5_USE_FILE_LOCKING']='FALSE' # just in case
os.environ['HDF5_MPI_OPT_TYPES']='TRUE'     # just in case


print('---------------------------------------------------------------------------------------------')
def main(date_to_parse, save_dir='/tmp'): # the main data crunching program


    data_files = ['~/search/loggerpcmount/albedo_slow.dat', '~/search/loggerpcmount/tracker_slow.dat']
    #data_files = ['./albedo_slow.dat', './tracker_slow.dat']

    header_length = 5

    df_list = []
    for datf in data_files: 
        
        day_delta = pd.to_timedelta(86399999999,unit='us')
        
        # create a dataframe to select rows using datetime objects, easier than text processing w/sed or something
        df               = get_csv_as_dataframe(datf)                # open the file as df for easy processing
        df['row_number'] = range(0,len(df))                          # assign a "row_number" column to keep track of entries
        df               = df[date_to_parse:date_to_parse+day_delta] # get up to and including only this day

        # get file row numbers corresponding to the first and last entry on the requested day
        start_row = df['row_number'].iat[0]+header_length-1
        end_row   = df['row_number'].iat[-1]+header_length

        # open file and get the selected
        with open(datf, 'r')  as f: flines = f.readlines()
        header         = flines[0:header_length-1]
        todays_entries = flines[start_row:end_row]
        save_lines     = header+todays_entries

        # save with original name, but append date
        save_str = f"{save_dir}/{os.path.basename(datf).split('.')[0]}_{date_to_parse.strftime('%Y%m%d')}.dat"

        print(f"\nSaving data from {datf} between \n{date_to_parse} ————> {date_to_parse+day_delta} to: ")
        print(f"         {save_str}\n")
        print(df)
        print('---------------------------------------------------------------------------------------------')
        with open(save_str, 'w') as f: f.writelines(save_lines)

    return df_list


def get_csv_as_dataframe(data_file):

    with open(data_file, 'r') as f:
        firstline = f.readlines()[1].rsplit(',')
        num_cols  = len(firstline)
        na_vals   = ['nan','NaN','NAN','NA','\"INF\"','\"-INF\"','\"NAN\"','\"NA','\"NAN','inf','-inf''\"inf\"','\"-inf\"']
        frame     = pd.read_csv(data_file, parse_dates=[0], sep=',', na_values=na_vals,\
                                index_col=0, header=[1], skiprows=[2, 3], engine='c')
 
    return frame.sort_index()


# this runs the function main as the main program... this is a hack that allows functions
# to come after the main code so it presents in a more logical, C-like, way
if __name__ == '__main__':

    # get some command line options
    parser = argparse.ArgumentParser()

    parser.add_argument('-d', '--datetoparse', metavar='str', help='day that you would like to pull out, YYYYMMDD')
    parser.add_argument('-s', '--savedir', metavar='str', help='directory to save output files to')

    args = parser.parse_args()

    if args.datetoparse: date_to_parse = datetime.strptime(args.datetoparse, '%Y%m%d')
    else: # default to yesterday if no arguments
        date_to_parse = datetime.today() - timedelta(1)
        date_to_parse = date_to_parse.replace(hour=0, minute=0, second=0, microsecond=0, day=date_to_parse.day)

    if args.savedir: save_dir = args.savedir
    else: save_dir = '/tmp/'

    main(date_to_parse, save_dir) # run it!
